#include <stdio.h>
#include <errno.h>
#include <fcntl.h>

/* @(#) pretty.c 1.1@(#) */

#include "health.h"

void usage(){
	puts("This program will open up /dev/health, and attempt to read");
	puts("temperature and fan speeds of your computer");
	puts("temperature #0 and #1 are usually cpu temps");
	puts("temperature #2 is the overall temperature");
}

void printfan(int fannum, int rpm, int div){
	double realrpm;
	printf("fan #%d:", fannum);
	if(rpm==0xff){
		realrpm=0;
	} else {
		realrpm= 1.35E6/(rpm*(1<<div)) ;
	}
	printf(" %f\n", realrpm);
}

int printtemp(int cpunum, int rawtemp)
{
	double celcius=(double)rawtemp;
	double far;
	
	/* adjustment for larger 9-bit values */
	if((cpunum >0) && ((rawtemp &&0x01) ==1)){
		celcius+=0.5;
	}
	far=celcius* 9 / 5 + 32;
	printf("temperature  #%d - %fC, %fF\n", cpunum, celcius, far);
}

int printvcore(int rawcore){
	double coreval;
	switch(rawcore){
		case 0 :  coreval=2.05; break; 
		case 1 : { coreval=2.00; break; }
		case 2 : { coreval=1.95; break; }
		case 3 : { coreval=1.90; break; }
		case 4 : { coreval=1.85; break; }
		case 5 : { coreval=1.80; break; }
		case 16 : { coreval=3.5; break; }
		case 17 : { coreval=3.4; break; }
		case 18 : { coreval=3.3; break; }
		case 19 : { coreval=3.2; break; }
		case 20 : { coreval=3.1; break; }
		case 21 : { coreval=3.0; break; }
		case 22 : { coreval=2.9; break; }
		case 23 : { coreval=2.8; break; }
		case 24 : { coreval=2.7; break; }
		case 25 : { coreval=2.6; break; }
		case 26 : { coreval=2.5; break; }
		case 27 : { coreval=2.4; break; }
		case 28 : { coreval=2.3; break; }
		case 29 : { coreval=2.2; break; }
		case 30 : { coreval=2.1; break; }
		case 31:
		default : { coreval=0.0; break; } 
	}
	printf("Vcore voltage is %f\n", coreval);
}
main(int argc, char *argv[]){
	int fd;
	struct rawdata data;

	if(argc!=1){
		usage();
		exit(1);
	}

	fd=open("/dev/health", O_RDONLY);
	if(fd==-1){
		perror("cannot open dev");
		exit(errno);
	}

	if(read(fd, &data, sizeof(data)) <=0){
		perror("error reading from dev");
	}
	printtemp(0,data.cputemp[0]);
	printtemp(1,(data.cputemp[1] >>1));
	printtemp(2,(data.cputemp[2]>>1));

	printfan(0, data.fanrpm[0], data.fandiv[0]);
	printfan(1, data.fanrpm[1], data.fandiv[1]);
	printfan(2, data.fanrpm[2], data.fandiv[2]);

	printvcore(data.vcore);
	puts("Skipping other voltage stuff for now");
}
